@@ -1,7 +1,4 @@
 package PEF::Front::Route;
-
-use strict;
-use warnings;
 use Scalar::Util qw(blessed);
 use PEF::Front::Config;
 use PEF::Front::Request;
@@ -13,6 +10,8 @@ use if cfg_handle_static(), 'File::LibMagic';
 use Encode;
 use URI::Escape;
 use base 'Exporter';
+use strict;
+use warnings;
 
 our @EXPORT = qw(get post patch put delete trace websocket sse);
 
@@ -197,19 +196,19 @@ sub rewrite_route {
 }
 
 sub prepare_context {
-	my $request = $_[0];
-	my $form    = $request->params;
+	my ($request, $prefix) = @_;
+	my $form = $request->params;
 	my $lang;
 	my ($src, $method, $params);
 	if (cfg_url_contains_lang) {
-		($lang, $src, $method, $params) = $request->path =~ m{^/(\w{2})/(app|ajax|submit|get)([^/]+)/?(.*)$};
+		($lang, $src, $method, $params) = $request->path =~ m{^/(\w{2})/(\Q$prefix\E)([^/]+)/?(.*)$};
 		if (not defined $lang) {
 			my $http_response = PEF::Front::Response->new(request => $request);
 			$http_response->redirect(cfg_location_error, 301);
 			return $http_response;
 		}
 	} else {
-		($src, $method, $params) = $request->path =~ m{^/(app|ajax|submit|get)([^/]+)/?(.*)$};
+		($src, $method, $params) = $request->path =~ m{^/(\Q$prefix\E)([^/]+)/?(.*)$};
 		if (not defined $method) {
 			my $http_response = PEF::Front::Response->new(request => $request);
 			$http_response->redirect(cfg_location_error, 301);
@@ -279,6 +278,13 @@ our %handlers = (
 	'/submit' => \&PEF::Front::Ajax::handler,
 );
 
+sub add_prefix_handler {
+	my ($prefix, $handler) = @_;
+	$prefix = '/' . $prefix if substr($prefix, 0, 1) ne '/';
+	die "bad handler type" if ref $handler ne 'CODE';
+	$handlers{$prefix} = $handler;
+}
+
 sub process_request {
 	my ($request, $parent_context) = @_;
 	cfg_log_level_info
@@ -299,14 +305,16 @@ sub process_request {
 	}
 	my $lang_offset = (cfg_url_contains_lang) ? 3 : 0;
 	my $handler;
+	my $handler_prefix;
 	for my $prefix (keys %handlers) {
 		if (substr($request->path, $lang_offset, length $prefix) eq $prefix) {
-			$handler = $handlers{$prefix};
+			$handler        = $handlers{$prefix};
+			$handler_prefix = $prefix;
 			last;
 		}
 	}
 	if ($handler) {
-		my $context = prepare_context($request);
+		my $context = prepare_context($request, $handler_prefix);
 		if (blessed($context) && $context->isa('PEF::Front::Response')) {
 			return $context->response();
 		}
@@ -373,6 +381,15 @@ It's like C</submit$Method> but by default parses parameters from URI path.
 
 =back
 
+=head2 CUSTOM PRFIXES
+
+=head3 B<add_prefix_handler($prefix, $handler)>
+
+Application can define its own prefix handlers B</$prefix$Method>. 
+C<$handler> is a code reference that receives C<($request, $context)>.
+
+=head2 RULES
+
 Using routing rules this schema can be changed completely. You can import
 your routing rules in  C<PEF::Front::Route> or add them  with
 C<PEF::Front::Route::add_route(@rules)>. 
@@ -499,7 +516,7 @@ request. C<PEF::Front::Route->to_app()> is this reference. Your last
 line in startup file usually should be
 
   PEF::Front::Route->to_app(); 
-
+  
 =head1 AUTHOR
  
 This module was written and is maintained by Anton Petrusevich.
